--[[ Text substitutions ]]
local L = MacroTalk_Locals

local subsBase = {
	["%%[Nn]"] = function() return UnitName("player") end,
	["%%[Zz]"] = GetRealZoneText,
	["%%[Ss][Zz]"] = GetSubZoneText,
	["%%[Ll][Oo][Cc]"] = function()
		local x, y = GetPlayerMapPosition("player")
		return ("%.0f, %.0f"):format(x * 100, y * 100)
	end,
}

local sexes = {
	L["<no %s>"]:format(L["gender"]),
	MALE:lower(),
	FEMALE:lower()
}

local unitInfoUnits = {
	[""] = "player",
	["[Tt]"] = "target",
	["[Ff]"] = "focus",
	["[Mm]"] = "mouseover",
	["[Pp]"] = "pet",
	["[Tt][Tt]"] = "targettarget",
}

local function ActualValue(unit)
	return UnitInParty(unit) or UnitInRaid(unit) or 
		(MacroTalk_InBattleground() and 
			UnitFactionGroup(unit) == UnitFactionGroup("player"))
end

local unitInfoPrefixes = {
	[""] = UnitName,
	
	["[Ll]"] = function(unit)
		local level = UnitLevel(unit)
		return level > 0 and level or level < 0 and "??"
	end,
	
	["[Cc]"] = UnitClass,
	
	["[Gg]"] = function(unit)
		return sexes[UnitSex(unit)]
	end,
	
	["[Rr]"] = function(unit)
		local race
		if UnitIsPlayer(unit) then
			race = UnitRace(unit)
		else
			race = UnitCreatureType(unit)
		end
		return race or L["<no %s>"]:format(RACE:lower())
	end,
	
	["[Hh]"] = function(unit)
		if ActualValue(unit) then
			return UnitHealth(unit).."/"..UnitHealthMax(unit)
		else
			return ("%.0f%%%%"):format(UnitHealth(unit))
		end
	end,
	
	["[Hh][Pp]"] = function(unit)
		return ("%.0f%%%%"):format(UnitHealth(unit)/UnitHealthMax(unit)*100)
	end,
	
	["[Mm][Aa]"] = function(unit)
		return UnitMana(unit).."/"..UnitManaMax(unit)
	end,
	
	["[Mm][Pp]"] = function(unit)
		local max = UnitManaMax(unit)
		if max > 0 then
			return ("%.0f%%%%"):format(max > 0 and UnitMana(unit)/max*100 or 0)
		else
			return L["<no %s>"]:format(MANA:lower())
		end
	end,
}

for initial, unit in pairs(unitInfoUnits) do
	local noUnit = L["<no %s>"]:format(unit)
	for suffix, func in pairs(unitInfoPrefixes) do
		if initial ~= "" and prefix ~= "" then
			local code = "%%"..initial..suffix
			if subsBase[code] then
				error(L["MacroTalk: Conflicting substitutions: "..code])
			end
			subsBase[code] = function()
				return UnitExists(unit) and func(unit) or noUnit
			end
		end
	end
end

local substitutions = {}
local i = 1
for k, v in pairs(subsBase) do
	substitutions[i] = { code = k, func = v }
	i = i + 1
end

sort(substitutions, function(subs1, subs2)
	return subs2.code:len() < subs1.code:len()
end)

local OrigSendChatMessage = SendChatMessage
function SendChatMessage(text, ...)
	for _, substitution in ipairs(substitutions) do
		text = text:gsub(substitution.code, substitution.func())
	end
	return OrigSendChatMessage(text, ...)
end
